var NMOTORS=3;
var state = new State;
var x = 0;

var defaultRatios=[1,0.5,0.25];

// Setup temperature pages
for(var e=0; e<2; e++) {
    var m = []
    var s = [];
    for(var i=0; i<12; i++) {
        m[i] = Math.random() > 0.5;
        s[i]=true;
    }
    project.setTag('e' + e + '_m', m, -1);
    project.setTag('e' + e + '_s', s, -1);
}

// Setup temperature monitoring
project.setTag('e0_t', 1);
project.setTag('e1_t', 1);

// Default values
for(var i=0; i<3; i++) {
    project.setTag('motor_ratio',defaultRatios[i],i);
    project.setTag('motor_on', 1, i);
    project.setTag('motor_setpoint', 55*defaultRatios[i], i);
}

// preconfigure temp setpoints for Overview page
for(var e=0; e<2; e++) {
    var S = project.getTag('e' + e + '_s', state, -1);
    var tOn = project.getTag('e' + e + '_t', state);
    var sp = [];
    for(var i=0; i<12; i++) {
        if (tOn && S[i]) { // general switch + local switch must be on
            var s = 150 * Math.random();
            sp[i] = s;
        } else {
            sp[i] = 0;
        }
    }
    project.setTag('e' + e + '_sp', sp, -1);
}
            

// Aggregate ctrl button press requests from different sources
// setting the unique ctrlRequest flag
project.aggregateCtrlRequests =  function()
{
    var v = project.getTag('ctrlRequestAlarm', state, 0);
    if (!v) {
        for(var i=0; i<NMOTORS && v == false; i++) {
            v = project.getTag('ctrlRequestMotor', state, i);
        }
        if (!v) {
            v = project.getTag('ctrlRequestTemp', state, 0);
        }
    }
    if (v) {
        project.setTag('ctrlRequest', 1);
    } else {
        project.setTag('ctrlRequest', 0);
    }
}

project.GefranMotorClass = function(wgt)
{
    this.wgt = wgt;
    // Bind js code to internal widget
    wgt.js = this;
    this.btnOnButton = wgt.getWidget(wgt.id + '.GBtn-on');
    this.btnOffButton = wgt.getWidget(wgt.id + '.GBtn-off');
    this.motorId = wgt.getWidget(wgt.id + ".motorId").value;
    this.onControl = function() {
        if (1 == this.btnOnButton.value ) {
            project.setTag('motor_on', 1, this.motorId);
        } else if (1 == this.btnOffButton.value ) {
            project.setTag('motor_on', 0, this.motorId);
        }
        // force button release and reset all ctrl requests
        this.btnOnButton.value = 0;
        this.btnOffButton.value = 0;
        project.setTag('ctrlRequestMotor', 0, this.motorId);
    }
    this.postCtrlRequest = function()
    {
        project.setTag('ctrlRequestMotor', 1, this.motorId);
    }
    this.clearCtrlRequest = function()
    {
        project.setTag('ctrlRequestMotor', 0, this.motorId);
    }
}

project.simulation = function()
{
    var syncMode = project.getTag('sync', state);
    var masterSetPoint = project.getTag('motor_setpoint', state, 0);
    var singleMode = project.getTag('single', state);
    
    var rand = [ Math.random(), Math.random(), Math.random() ];
    // simulate setpoint
    for(var i=0; i<NMOTORS; i++) {
        var on = project.getTag('motor_on', state, i);
        var r = rand[i];
        var setPoint = project.getTag('motor_setpoint', state, i);
        if (syncMode && i!=0) { // i == 0 is the master
            var ratio = project.getTag('motor_ratio', state, i);
            var syncSetPoint = ratio * masterSetPoint;
            // reflect changes from master to slave setpoint
            if (syncSetPoint != setPoint && !singleMode) {
                project.setTag('motor_setpoint', syncSetPoint, i);
            }
        } else {
            setPoint = project.getTag('motor_setpoint', state, i);
        }
        if (on) { // motor running
            var curPoint = project.getTag('motor_rpm', state, i);
            var step = (setPoint-curPoint)/2 + 3*(r-0.5); /* noise */
            if (step != 0) {
                var randomStep = step*(r<0.7) + (step/2)*(r>0.7);
                project.setTag('motor_rpm', Math.max(0, curPoint + randomStep), i);
            }
            // amperes
            project.setTag('motor_ampere', 20 + 10*(r-0.5), i);
            project.setTag('motor_bar', 100 + 30*(r-0.5), i);
            project.setTag('motor_celsius', 150 + 20*(r-0.5), i);
        } else { // motor off
            project.setTag('motor_rpm', 0, i);
            project.setTag('motor_ampere', 0, i);
            project.setTag('motor_bar', 0, i);
            project.setTag('motor_celsius', 0, i);
        }
    }

    if (syncMode && singleMode) {
        // recompute ratios
        for(var i=1; i<3; i++) {
           var sp = project.getTag('motor_setpoint',state, i);
           project.setTag('motor_ratio', sp / masterSetPoint, i);
        }
    }
}

// start motor simulation
project.setInterval(project.simulation, 500);





project.trendSimulation = function ()
{ 
   
            x = x + 0.1;
            project.setTag("plot_0", 50+40*Math.sin(x)+3*Math.random());
            project.setTag("plot_1", 20+15*Math.sin(x)+3*Math.random());
            project.setTag("plot_2", 75+22*Math.sin(x)+3*Math.random());
  
  
}

// start trend simulation
project.trendSimulation();
project.setInterval(project.trendSimulation, 900);

